/**
 * WorldController.java
 *
 *
 * Created: Tue Jul 04 07:16:11 2000
 *
 * @author 
 * @version
 */

package algae.simulation;

import java.util.*;

import algae.simulation.Settings;
import algae.simulation.worlds.*;
import algae.simulation.worlds.PheromoneWorld;
import algae.simulation.worlds.SubstrateWorld; 
import algae.simulation.agents.*;
import algae.simulation.view.*;
import algae.gui.MainGUI;

// WorldController version for GUI usage
public class GUIWC extends WC {
    
    // Clients to view the world's state in
    MainGUI alg;
     
    // the views
    ViewAgent aView;

    public GUIWC(Settings simulationSettings, MainGUI p) {
	// set the GUI where the simulation is viewed
	super(simulationSettings);
	alg = p;
    }
    
    public WorldState getState()
    {
    	return (new WorldState(this));
    }

    public WorldState run()
    {
	if (stepsRunning<stepsToRun)
	{
	    step();
	    WorldState ws=new WorldState(this);
	    if (ws==null)
	       System.out.println("ws==null");
	    return (ws);
	}
	else
	{
	    int tmp=alg.likeToContinue(stepsRunning);
	    if (tmp!=0)
		stepsToRun+=tmp;
	    else
		finish();
        }
	return null;
    }

    public int getAgentAt(int px, int py)
    {
    	Agent tmp;
    	for (int z=0; z<agents.agentCount; z++)
    	{
    	    tmp=(Agent)agents.agents.elementAt(z);
    	    if ((tmp.x==px)&&(tmp.y==py)&&(tmp.isAlive()))
    	       return tmp.agentID;
    	}
    	return -1;
    }
    
    public Agent getAgentByID(int n)
    {
    	int idx=agents.getAgentIndexByID(n);
    	Agent tmp=(Agent)agents.agents.elementAt(idx);
    	return tmp;
    }
    
    public float getLightAt(int px, int py)
    {
    	return (float)light.world[py][px].amount;
    }

    public float getSubstrateAt(int px, int py)
    {
    	return (float)substrate.world[py][px].amount;
    }

    public float getPheromoneAt(int px, int py)
    {
    	return (float)pheromone.world[py][px].amount;
    }

    public float[] getAgentInfos(int n)
    {
    	int idx=agents.getAgentIndexByID(n);
    	Agent tmp=(Agent)agents.agents.elementAt(idx);
    	
    	return tmp.getInfos();
    }
    
    
} // WorldController
