package algae.simulation;

import java.util.*;

import algae.simulation.Settings;
import algae.simulation.worlds.*;
import algae.simulation.worlds.PheromoneWorld;
import algae.simulation.worlds.SubstrateWorld; 
import algae.simulation.agents.*;
import algae.gui.MainGUI;
import algae.simulation.SimulationStats;
import algae.simulation.SinCosLut;

// abstract class for worldcontrollern
public abstract class WC {

    // Attributes
    public int stepsToRun=0;
    public int stepsRunning=0;
    
    // the simulations settings
    public Settings cfg;
  
    // the worlds
    public AgentWorld agents;
    public SubstrateWorld substrate;
    public PheromoneWorld pheromone;
    public LightWorld light;

    public SimulationStats stats;

    SinCosLut lut= new SinCosLut();
   
    public WC(Settings set)
    {
    	cfg=set;
    	
    	// initialize the simulation
	stepsToRun=cfg.duration;
	
	substrate=new SubstrateWorld(cfg);
	pheromone=new PheromoneWorld(cfg,lut);
	light=new LightWorld(cfg);
	agents=new AgentWorld(cfg,substrate,pheromone,light);
	light.setAgentWorld(agents);
	
	stepsRunning=0;
    }

    public void step()
    {
	stepsRunning++;
	     
    	substrate.step(stepsRunning);
	pheromone.step(stepsRunning);
	light.step(stepsRunning);
	agents.step(stepsRunning);
		
    }
    
    public void finish()
    {
	//this=null;
    }
}
