package algae.xmlstuff;

import algae.simulation.Settings;

import java.io.*;
import java.util.*;
import java.lang.Short;
import java.util.Date;
import java.text.DateFormat;
import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.*;

public class SettingsImport {
	
     public SettingsImport() 
     {
     }

     public Settings importSettingsFile(String fileName)
     {
     	Settings cfg=new Settings();

        File xmlfile=new File(fileName);
	
	if (!xmlfile.exists())
	{
	     System.out.println("File "+fileName+" not found ...");
	     return null;
	}

    	System.out.println("Trying to parse \""+fileName+"\"");
    	
        XmlDocument xmldoc = new XmlDocument();
        // Load the File
        try 
        {
            InputSource xmlinput = Resolver.createInputSource (xmlfile);
            xmldoc = XmlDocument.createXmlDocument (xmlinput, false);
        }
         
        // Do finer catches !!!!!
         
        catch (SAXParseException err) 
        {   
               System.out.println ("** Parsing error" 
                                   + ", line " + err.getLineNumber ()
                                   + ", uri " + err.getSystemId ());
               System.out.println("   " + err.getMessage ());
        } 
         
        catch (SAXException ex) 
        {
            Exception   x = ex.getException ();
            ((x == null) ? ex : x).printStackTrace ();
        }
            
        catch (Throwable t) 
        {
            t.printStackTrace ();
        }   
      	
      	//// get Root XmlNode
        org.w3c.dom.Node xmlRoot = xmldoc.getDocumentElement();
        NodeList rootChilds = xmlRoot.getChildNodes();
        System.out.println("Rootnodename = "+xmlRoot.getNodeName());
        for (int i=0; i<rootChilds.getLength(); i++) 
        {
            if (rootChilds.item(i).getNodeName().equals("Algaes")) 
            {
            	ImportAlgaesSettings(rootChilds.item(i), cfg);
               	System.out.println("Algaes-subsection imported...");
            }
            else 
            if (rootChilds.item(i).getNodeName().equals("Actions")) 
            {
            	ImportActionsSettings(rootChilds.item(i), cfg);
               	System.out.println("Actions-subsection found...");
            }
            else 
            if (rootChilds.item(i).getNodeName().equals("Simulation")) 
            {
            	ImportSimulationSettings(rootChilds.item(i), cfg);
               	System.out.println("Simulation-subsection found...");
            }
            else 
            if (rootChilds.item(i).getNodeName().equals("Sensors")) 
            {
            	ImportSensorsSettings(rootChilds.item(i), cfg);
               	System.out.println("Sensors-subsection found...");
            }
            else 
            if (rootChilds.item(i).getNodeName().equals("ANN")) 
            {
            	ImportANNSettings(rootChilds.item(i), cfg);
               	System.out.println("ANN-subsection found...");
            }
            else 
            if (rootChilds.item(i).getNodeName().equals("Light")) 
            {
            	ImportLightSettings(rootChilds.item(i), cfg);
               	System.out.println("Light-subsection found...");
            }
            else 
            if (rootChilds.item(i).getNodeName().equals("Substrate")) 
            {
            	ImportSubstrateSettings(rootChilds.item(i), cfg);
               	System.out.println("Substrate-subsection found...");
            }
	    else 
            if (rootChilds.item(i).getNodeName().equals("Evolution")) 
            {
            	ImportEvolutionSettings(rootChilds.item(i), cfg);
               	System.out.println("Evolution-subsection found...");
            }
            else System.out.println("Import: Unknown node in XML file (" + rootChilds.item(i).getNodeName() + ")!");
	}
	
	return cfg;
     }

     static void ImportSimulationSettings(org.w3c.dom.Node xn, Settings cfg)
     {
     	try {
    	   cfg.duration=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("duration").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.duration=-1;
	}
	try {
           cfg.width=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("width").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.width=-1;
	}
	try {
	   cfg.height=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("height").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.height=-1;
	}
	
	try {
    	   cfg.randSeed=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("randseed").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.randSeed=0x0cafe; // ;))
	}
	
	try {
    	   cfg.pheromone_dec_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("pheromone_dec_rate").getNodeValue()); 	
    	} catch (NullPointerException e)
	{
	   cfg.pheromone_dec_rate=-1.0f;
	}
    }

    static void ImportAlgaesSettings(org.w3c.dom.Node xn, Settings cfg)
    {
    	try {
    	   cfg.initial_algaes=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("initial_algaes").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.initial_algaes=-1;
	}  
	
	try { 
	   cfg.initial_energy=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("initial_energy").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.initial_energy=-1.0f;
	}
	try {
    	   cfg.agent_transluency=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("agent_transluency").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.agent_transluency=-1.0f;
	}
    	try {
    	   cfg.pheromone_radius=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("pheromone_radius").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.pheromone_radius=-1.0f;
	}
    }

    static void ImportActionsSettings(org.w3c.dom.Node xn, Settings cfg)
    {
    	try {
   	   cfg.energy_movement=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("energy_movement").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.energy_movement=-1.0f;
	}
    	try {
   	   cfg.energy_mitose=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("energy_mitose").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.energy_mitose=-1.0f;
	}
    	try {
   	   cfg.energy_meiose=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("energy_meiose").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.energy_meiose=-1.0f;
	}
    	try {
   	   cfg.energy_pheromone_output=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("energy_pheromone_output").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.energy_pheromone_output=-1.0f;
	}
     	try {
   	   cfg.energy_substrate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("energy_substrate").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.energy_substrate=-1.0f;
	}
    	try {
   	   cfg.energy_light=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("energy_light").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.energy_light=-1.0f;
	}
    	try {
   	   cfg.energy_standstill=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("energy_standstill").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.energy_standstill=-1.0f;
	}
    	
    	try {
   	   cfg.min_energy_mitose=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("min_energy_mitose").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.min_energy_mitose=-1.0f;
	}
    	try {
   	   cfg.min_energy_meiose=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("min_energy_meiose").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.min_energy_meiose=-1.0f;
	}
    	try {
   	   cfg.min_energy_pheromone_output=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("min_energy_pheromone_output").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.min_energy_pheromone_output=-1.0f;
	}

    	try {
   	   cfg.max_pheromone_saturation=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("max_pheromone_saturation").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.max_pheromone_saturation=-1.0f;
	}
    	try {
   	   cfg.max_substrate_saturation=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("max_substrate_saturation").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.max_substrate_saturation=-1.0f;
	}
	
	try {
   	   cfg.bumper_sensor_efficiency=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("bumper_sensor_efficiency").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.bumper_sensor_efficiency=-1;
	}
	try {
   	   cfg.pheromone_sensor_efficiency=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("pheromone_sensor_efficiency").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.pheromone_sensor_efficiency=-1;
	}
	try {
   	   cfg.substrate_sensor_efficiency=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("substrate_sensor_efficiency").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.substrate_sensor_efficiency=-1;
	}
	try {
   	   cfg.light_sensor_efficiency=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("light_sensor_efficiency").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.light_sensor_efficiency=-1;
	}
	try {
   	   cfg.substrate_metabolic_efficiency=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("substrate_metabolic_efficiency").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.substrate_metabolic_efficiency=-1;
	}
	try {
   	   cfg.light_metabolic_efficiency=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("light_metabolic_efficiency").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.light_metabolic_efficiency=-1;
	}
	try {
	   cfg.sensor_sensitivities=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("sensor_sensitivities").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.sensor_sensitivities=0;
	}
	try {
	   cfg.metabolic_markers=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("metabolic_markers").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.metabolic_markers=0;
	}
    }

    static void ImportSensorsSettings(org.w3c.dom.Node xn, Settings cfg)
    {
    	try {
     	   cfg.pheromone_fov=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("pheromone_fov").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.pheromone_fov=-1;
	}
    	try {
    	   cfg.pheromone_range=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("pheromone_range").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.pheromone_range=-1;
	}
    	
    	try {
    	   cfg.substrate_fov=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("substrate_fov").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.substrate_fov=-1;
	}
    	try {
    	   cfg.substrate_range=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("substrate_range").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.substrate_range=-1;
	}
    	
    	try {
    	   cfg.light_fov=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("light_fov").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.light_fov=-1;
	}
    	try {
    	   cfg.light_range=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("light_range").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.light_range=-1;
	}
    	
    	try {
    	   cfg.bumper_range=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("bumper_range").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.bumper_range=-1;
	}
    }
    
    static void ImportANNSettings(org.w3c.dom.Node xn, Settings cfg)
    {
    	try {
    	   cfg.number_of_neurons=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("number_of_neurons").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.number_of_neurons=-1;
	}
    	try {
	   cfg.initial_ann_bias=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("initial_ann_bias").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.initial_ann_bias=-1.0f;
	}
    	try {
	   cfg.spline_node_count=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("cubic_spline_nodecount").getNodeValue());
    	} catch (NullPointerException e)
	{
	   cfg.spline_node_count=-1;
	}
	try {
	   String tmp=xn.getAttributes().getNamedItem("ann_transfer_func").getNodeValue();
	   if (tmp.equals("tan")) cfg.ann_transfer_func=1;
	   if (tmp.equals("sin")) cfg.ann_transfer_func=2;
	   if (tmp.equals("cos")) cfg.ann_transfer_func=3;
	   if (tmp.equals("atan")) cfg.ann_transfer_func=4;
	   if (tmp.equals("asin")) cfg.ann_transfer_func=5;
	   if (tmp.equals("acos")) cfg.ann_transfer_func=6;
	} catch (NullPointerException e)
	{
	   cfg.ann_transfer_func=1;
	}
    }
    
    static void ImportEvolutionSettings(org.w3c.dom.Node xn, Settings cfg)
    {
	try {
	   cfg.crossover_type=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("crossover_type").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.crossover_type=-1;
	}
	try {
	   cfg.ann_mutation_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("ann_mutation_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.ann_mutation_rate=-1.0f;
	}
	try {
	   cfg.ann_crossover_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("ann_crossover_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.ann_crossover_rate=-1.0f;
	}
	try {
	   cfg.cs_mutation_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("cs_mutation_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.cs_mutation_rate=-1.0f;
	}
	try {
	   cfg.cs_crossover_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("cs_crossover_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.cs_crossover_rate=-1.0f;
	}
    
        try {
	   cfg.ss_mutation_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("ss_mutation_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.ss_mutation_rate=-1.0f;
	}
	try {
	   cfg.ss_crossover_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("ss_crossover_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.ss_crossover_rate=-1.0f;
	}
    
    
    	try {
	   cfg.se_mutation_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("se_mutation_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.se_mutation_rate=-1.0f;
	}
	try {
	   cfg.se_crossover_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("se_crossover_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.se_crossover_rate=-1.0f;
	}
	try {
	   cfg.me_mutation_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("me_mutation_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.me_mutation_rate=-1.0f;
	}
	try {
	   cfg.me_crossover_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("me_crossover_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.me_crossover_rate=-1.0f;
	}
	try {
	   cfg.mm_mutation_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("mm_mutation_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.mm_mutation_rate=-1.0f;
	}
	try {
	   cfg.mm_crossover_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("mm_crossover_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.mm_crossover_rate=-1.0f;
	}
    }

    static void ImportLightSettings(org.w3c.dom.Node xn, Settings cfg)
    {
    	try {
    	   if (xn.getAttributes().getNamedItem("light_enabled").getNodeValue().equals("all the time"))
    	       cfg.light_enabled=1;
    	   if (xn.getAttributes().getNamedItem("light_enabled").getNodeValue().equals("function"))
    	       cfg.light_enabled=2;
    	   if (xn.getAttributes().getNamedItem("light_enabled").getNodeValue().equals("never"))
    	       cfg.light_enabled=0;
	} catch (NullPointerException e)
	{
	   cfg.light_enabled=0;
	}
	
	try {
	   if (xn.getAttributes().getNamedItem("light_distribution_func").getNodeValue().equals("1/y"))
	       cfg.light_function=1;
	   else
	       cfg.light_function=0; // stored func is not defined
	} catch (NullPointerException e)
	{
	   cfg.light_function=0;
	}
	
	try {
	   if (xn.getAttributes().getNamedItem("light_cycle_func").equals("sin(t)"))
    	       cfg.light_cycle_function=1;
	   else
	   if (xn.getAttributes().getNamedItem("light_cycle_func").equals("rectangle(t)"))
    	       cfg.light_cycle_function=2;
	   else
	   if (xn.getAttributes().getNamedItem("light_cycle_func").equals("chainsaw(t)"))
    	       cfg.light_cycle_function=3;
           else
	   if (xn.getAttributes().getNamedItem("light_cycle_func").equals("zigzag(t)"))
    	       cfg.light_cycle_function=4;
    	   else
    	       cfg.light_cycle_function=0; // stored func is not defined    
	} catch (NullPointerException e)
	{
	   cfg.light_cycle_function=0;
	}

	try {
    	   cfg.light_intensity=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("light_intensity").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.light_intensity=-1.0f;
	}
    }

    static void ImportSubstrateSettings(org.w3c.dom.Node xn, Settings cfg)
    {
    	try {
	   cfg.substrate_amount=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("substrate_amount").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.substrate_amount=-1;
	}
    	try {
	   cfg.substrate_renew_rate=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("substrate_renew_rate").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.substrate_renew_rate=-1.0f;
	}
    	try {
	   cfg.substrate_for_dead_agent=java.lang.Float.parseFloat(xn.getAttributes().getNamedItem("substrate_for_dead_agent").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.substrate_for_dead_agent=-1.0f;
	}
    	try {
	   cfg.substrate_periodic_amount=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("periodic_substrate_amount").getNodeValue());		
	} catch (NullPointerException e)
	{
	   cfg.substrate_periodic_amount=-1;
	}
    	try {
	   cfg.substrate_periodic_steps=java.lang.Integer.parseInt(xn.getAttributes().getNamedItem("periodic_substrate_steps").getNodeValue());
	} catch (NullPointerException e)
	{
	   cfg.substrate_periodic_steps=-1;
	}
	
	// TODO:
	cfg.substrate_function=0;
	
    	try {
  	   if (xn.getAttributes().getNamedItem("periodic_substrate").getNodeValue().equals("1"))
	      cfg.periodic_substrate=1;
	   else
	      cfg.periodic_substrate=0;
	} catch (NullPointerException e)
	{
	   cfg.periodic_substrate=0;
	}
    }
}